package sample.textdetection;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.StringJoiner;

import org.apache.commons.codec.binary.Base64;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.util.EntityUtils;
import org.apache.poi.EncryptedDocumentException;
import org.apache.poi.openxml4j.exceptions.InvalidFormatException;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.apache.poi.xssf.usermodel.XSSFCell;
import org.apache.poi.xssf.usermodel.XSSFRow;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.databind.ObjectMapper;

import sample.textdetection.mapper.GCPBlock;
import sample.textdetection.mapper.GCPBoundingBox;
import sample.textdetection.mapper.GCPFeature;
import sample.textdetection.mapper.GCPFullTextAnnotation;
import sample.textdetection.mapper.GCPImage;
import sample.textdetection.mapper.GCPPage;
import sample.textdetection.mapper.GCPRequest;
import sample.textdetection.mapper.GCPResponse;
import sample.textdetection.mapper.GCPResultInfo;
import sample.textdetection.mapper.GCPSendInfo;

public class TextDetection {

	/* Google Cloud Vision API ̃T[rXGh|CgiURLj */
	public static final String uri = "https://vision.googleapis.com/v1/images:annotate";
	/* Google Cloud Vision APIL[ */
	 public static final String key = "쐬APIL[";
	/* ExceĺACfbNX0n܂ */
	public static final int OFFSET = 1;

	/**
	 * @param args
	 * @throws EncryptedDocumentException
	 * @throws InvalidFormatException
	 */
	public static void main(String[] args) throws EncryptedDocumentException, InvalidFormatException {
		if (args.length != 2) {
			System.err.println("ɂ̓tH_pXw肵ĂB");
			return;
		}
		String folderPath = args[0];
		String filePath = args[1];
		TextDetection detection = new TextDetection();
		try {
			detection.run(folderPath, filePath);
		} catch (IOException e) {
			e.printStackTrace();
		} catch (URISyntaxException e) {
			e.printStackTrace();
		}
	}

	/**
	 * eXg̓_
	 * @param folderPath eXg摜i[Cӂ̃tH_pX
	 * @param filePath ̓_ʋLpExcelt@CpX
	 * @throws IOException
	 * @throws URISyntaxException
	 * @throws InvalidFormatException
	 * @throws EncryptedDocumentException
	 */
	public void run(String folderPath, String filePath)
			throws IOException, URISyntaxException, EncryptedDocumentException, InvalidFormatException {
		/* wtH_̃t@CFileIuWFNgɕϊ */
		List<File> files = load(folderPath);
		/* HTTPʐMp̃NCAg쐬 */
		HttpClient httpclient = createClient();
		/* Request쐬 */
		HttpPost request = createRequest();

		/* DTONX쐬 */
		GCPSendInfo sendInfo = createSendDTO(files);
		/* DTONXJSON֕ϊ */
		ObjectMapper mapper = new ObjectMapper().setSerializationInclusion(JsonInclude.Include.NON_NULL);
		String body = mapper.writeValueAsString(sendInfo);

		/* NGXg{JSONݒ肷 */
		StringEntity reqEntity = new StringEntity(body);
		request.setEntity(reqEntity);
		/* 摜͈˗(NGXg)s */
		HttpResponse response = httpclient.execute(request);
		/* (X|X)f[^擾 */
		HttpEntity entity = response.getEntity();
		/* JSON̕ƂĎ擾 */
		String jsonString = EntityUtils.toString(entity).trim();

		/* JSON̕JavaIuWFNgɃ}bsO */
		GCPResultInfo resultInfo = mapper.readValue(jsonString, GCPResultInfo.class);

		List<DetectResult> results = createDetectResults(resultInfo);

		writeMarkResult(filePath, results);
	}

	/**
	 * ėp猟oExcelɏo
	 * @param templatePath
	 * @param results
	 * @throws EncryptedDocumentException
	 * @throws InvalidFormatException
	 * @throws IOException
	 */
	public void writeMarkResult(String templatePath, List<DetectResult> results)
			throws EncryptedDocumentException, InvalidFormatException, IOException {
		/* ̃ev[gpXǂݍ */
		FileInputStream in = new FileInputStream(templatePath);
		/* ݗpXg[` */
		FileOutputStream out = null;
		XSSFWorkbook srcWb = null;
		try {
			/* ExcelWorkbook쐬 */
			srcWb = (XSSFWorkbook) WorkbookFactory.create(in);
			/* WorkbookSheet1擾 */
			XSSFSheet sheet = srcWb.getSheet("Sheet1");
			/* l̏܂ĂŏIs擾Bf[^͂̎̍sȍ~ɑ} */
			int lastRow = sheet.getLastRowNum();
			/* l̏܂ĂŏI擾 */
			XSSFRow srcRow = sheet.getRow(0);
			int lastCol = srcRow.getLastCellNum();
			/* l̉𓚂 */
			for (int num = 0; num < results.size(); num++) {
				DetectResult result = results.get(num);
				/* l͂Ăs1ɐVs} */
				XSSFRow dstRow = sheet.createRow(++lastRow);
				/* Ɖ𓚋yюZɏ */
				for (int i = 0; i < lastCol; i++) {
					/* Z쐬 */
					XSSFCell cell = dstRow.createCell(i);
					if (i != 0 && (i % 2) == 0) {
						int colIdx = cell.getColumnIndex() - OFFSET;
						int rowIdx = cell.getRowIndex() + OFFSET;
						String col = String.valueOf((char) ('A' + colIdx));
						String formula = String.format("%s$2=%s%d", col, col, rowIdx);
						cell.setCellFormula(formula);
					} else {
						if (i == (lastCol - 1)) {
							int rowIdx = cell.getRowIndex() + OFFSET;
							String formula = String.format("COUNTIF(B%d:G%d, TRUE)", rowIdx, rowIdx);
							cell.setCellFormula(formula);
						} else {
							if (i == 0) {
								cell.setCellValue(result.getName());
							} else {
								cell.setCellValue(Integer.parseInt(result.getAnswers().get(i / 2)));
							}
						}
					}
				}
			}
			out = new FileOutputStream(templatePath);
			srcWb.write(out);
		} finally {
			if (srcWb != null)
				srcWb.close();
			if (out != null)
				out.close();
		}
	}

	/**
	 * w肵tH_pX̃t@Cǂݍ
	 * @param path Cӂ̃tH_pX
	 * @return
	 */
	public List<File> load(String path) {
		List<File> files = new ArrayList<>();
		/* tH_pXFileIuWFNgɕϊ */
		File folder = new File(path);
		if (folder.exists()) {
			/* tH_t@C擾 */
			for (File file : folder.listFiles()) {
				files.add(file);
			}
		}
		return files;
	}

	/**
	 * VisionAPĨNGXgf[^ɐݒ肷邽߂Ƀt@CBase64Stringɕϊ
	 * @param file 摜FΏۂ̃t@C
	 * @return
	 * @throws IOException
	 */
	public String encodeBase64String(File file) throws IOException {
		FileInputStream fis = new FileInputStream(file);
		byte[] bytes = new byte[(int) file.length()];
		fis.read(bytes);
		fis.close();
		return Base64.encodeBase64String(bytes);
	}

	/**
	 * HTTP Client 쐬
	 * @return
	 */
	public HttpClient createClient() {
		/* ^CAEgԂ̐ݒi~bj */
		int socketTimeout = 5000;
		int connectionTimeout = 5000;
		RequestConfig requestConfig = RequestConfig.custom().setConnectTimeout(connectionTimeout)
				.setSocketTimeout(socketTimeout).build();

		HttpClient httpclient = HttpClientBuilder.create().setDefaultRequestConfig(requestConfig).build();
		return httpclient;
	}

	/**
	 * Request쐬
	 * @return
	 * @throws URISyntaxException
	 */
	public HttpPost createRequest() throws URISyntaxException {
		/* Google Cloud Vision API URLPOSTNGXg𑗂 */
		URIBuilder builder = new URIBuilder(uri);
		/* p[^key, p[^l{APIL[} */
		builder.setParameter("key", key);
		URI uri = builder.build();
		HttpPost request = new HttpPost(uri);
		/* JSON`ł邱Ƃ𖾎 */
		request.setHeader("Content-Type", "application/json");
		return request;
	}

	/**
	 * NGXg{fB쐬
	 * @param files
	 * @return
	 * @throws IOException
	 */
	public GCPSendInfo createSendDTO(List<File> files) throws IOException {
		GCPSendInfo sendInfo = new GCPSendInfo();
		sendInfo.requests = new GCPRequest[files.size()];
		int i = 0;
		for (File file : files) {
			/* t@CƂɃNGXg쐬 */
			sendInfo.requests[i] = new GCPRequest();
			sendInfo.requests[i].image = new GCPImage();
			sendInfo.requests[i].image.content = encodeBase64String(file);
			sendInfo.requests[i].features = new GCPFeature[1];
			sendInfo.requests[i].features[0] = new GCPFeature();
			sendInfo.requests[i].features[0].type = "TEXT_DETECTION";
			i++;
		}
		return sendInfo;
	}

	/**
	 * 摜F̌ʂ疼OA𓚂𒊏o
	 * @param resultInfo
	 * @return
	 */
	public List<DetectResult> createDetectResults(GCPResultInfo resultInfo) {
		List<DetectResult> results = new ArrayList<>();
		for (GCPResponse res : resultInfo.responses) {
			int c = 0;
			DetectResult result = new DetectResult();
			GCPFullTextAnnotation anno = res.fullTextAnnotation;
			for (GCPPage page : anno.pages) {
				for (GCPBlock block : page.blocks) {
					GCPBoundingBox box = block.boundingBox;
					int x = box.vertices[0].x;
					int y = box.vertices[0].y;
					/* 臒lŒlƂBA𓚂݂̂͑ΏۂƂȂ悤ɐݒ */
					if (x < 800 || y > 1800)
						continue;
					StringJoiner sj = new StringJoiner("");
					Arrays.stream(block.paragraphs).flatMap(para -> Arrays.stream(para.words))
							.flatMap(word -> Arrays.stream(word.symbols)).forEach(symbol -> {
								sj.add(symbol.text);
							});
					String text = sj.toString();
					if (c > 0) {
						result.addAnswer(text);
					} else {
						result.setName(text);
					}
					c++;
				}
			}
			results.add(result);
		}
		return results;
	}

	public static class DetectResult {

		/*  */
		private String name;

		/* 𓚂̈ꗗ */
		private List<String> answers;

		public DetectResult() {
			this.answers = new ArrayList<String>();
		}

		public String getName() {
			return name;
		}

		public void setName(String name) {
			this.name = name;
		}

		public List<String> getAnswers() {
			return answers;
		}

		public void addAnswer(String answer) {
			this.answers.add(answer);
		}

		@Override
		public String toString() {
			return getName() + ":" + getAnswers();
		}
	}
}
